import argparse
import importlib.util
import os
import sys
from pathlib import Path
from typing import List

from colorama import Fore, Style
from rich.console import Console  # type: ignore  # pylint: disable=import-error
from rich.table import Table  # type: ignore  # pylint: disable=import-error

from vds.config import Config, ValidDataSetPlugin  # type: ignore
from vds.colorizer import Colorizer  # type: ignore


def parser() -> argparse.ArgumentParser:
    argument_parser = argparse.ArgumentParser(description='Validate audio and text files.')
    argument_parser.add_argument(
        '-p', '--path', type=Path, action='store', required=False, default='wavs',
        help='Path to dataset',
    )
    argument_parser.add_argument(
        '-d', '--disable', type=str, action='store', required=False, default='',
        help='Disable plugins',
    )
    argument_parser.add_argument(
        '-f', '--files', type=str, action='store', required=False, default='list_train.txt,list_val.txt',
        help='Set transcription file names',
    )
    argument_parser.add_argument('--list-plugins', action='store_true', help='List plugins')

    argument_parser.add_argument(
        '-v', '--verbose', action='store_true', required=False, default=False,
        help='Print additional information',
    )
    return argument_parser


def list_plugins() -> None:
    plugin_modules = get_plugins()

    table = Table(title='Plugins')

    table.add_column('ID', justify='center', style='cyan', no_wrap=True)
    table.add_column('Name', justify='left', style='magenta', no_wrap=True)
    table.add_column('Version', justify='center', style='yellow', no_wrap=True)
    table.add_column('Description', justify='left', style='green', no_wrap=True)

    for valid_dataset_plugin in plugin_modules:
        vds_plugin = valid_dataset_plugin.init_plugin()
        table.add_row(vds_plugin.info.id, vds_plugin.info.name, vds_plugin.info.version, vds_plugin.info.description)
    console = Console()
    console.print(table)
    print()


def get_plugins() -> List[ValidDataSetPlugin]:
    Config.disabled_plugins = [f'{file.lower()}.py' for file in Config.arguments.disable.split(',')]

    Config.transcription_plugins_dir_path = str(
        Path(__file__).parent / 'plugins' / Config.transcription_plugins_dir_path,
    )
    Config.files_plugins_dir_path = str(
        Path(__file__).parent / 'plugins' / Config.files_plugins_dir_path,
    )

    Config.activate_plugins = [
        file for file in (
            os.listdir(Path(__file__).parent / 'plugins' / Config.transcription_plugins_dir_path)
            + os.listdir(Path(__file__).parent / 'plugins' / Config.files_plugins_dir_path)
        ) if file.endswith('.py') and file not in Config.disabled_plugins
    ]

    plugin_modules = []

    for plugin_file in sorted(Config.activate_plugins):
        module_name: str = str(os.path.splitext(plugin_file)[0])
        module_path: str = ''
        if 't' in plugin_file:
            module_path = str(Path(Config.transcription_plugins_dir_path) / str(plugin_file))
        elif 'f' in plugin_file:
            module_path = str(Path(Config.files_plugins_dir_path) / str(plugin_file))

        spec = importlib.util.spec_from_file_location(str(module_name), module_path)
        if spec is None:
            continue
        plugin_module = importlib.util.module_from_spec(spec)
        if spec.loader is not None:
            spec.loader.exec_module(plugin_module)
        plugin_modules.append(plugin_module)
    return plugin_modules


def main() -> None:
    Config.arguments = parser().parse_args()

    if Config.arguments.list_plugins:
        list_plugins()
        sys.exit(0)

    colorizer = Colorizer()

    _status_ok = f'{Fore.GREEN}{Style.BRIGHT} OK {Style.RESET_ALL}'
    _status_fail = f'{Fore.RED}{Style.BRIGHT}FAIL{Style.RESET_ALL}'

    plugin_modules = get_plugins()

    for valid_dataset_plugin in plugin_modules:
        vds_plugin = valid_dataset_plugin.init_plugin()

        colored_id = f'{Fore.LIGHTRED_EX}{Style.BRIGHT}{vds_plugin.info.id}{Style.RESET_ALL}'

        vds_plugin.run(
            Config.arguments.path, tuple(Path(path) for path in str(Config.arguments.files).split(',')), 'wavs',
        )

        if len(vds_plugin.errors) == 0 and Config.arguments.verbose:
            print(f'{colored_id}: [{_status_ok}] {vds_plugin.success_message}\n')

        if len(vds_plugin.errors) > 0:
            print(f'{colored_id}: [{_status_fail}] {vds_plugin.error_message}\n')
            for error in vds_plugin.errors:
                print(colorizer.colorize(error))
            print()


if __name__ == '__main__':
    main()



from pathlib import Path
from typing import List, Tuple


class PluginInfo:
    author: str = 'Tadeusz Miszczyk'
    description: str = 'Check if the "wavs" folder and transcription files exist in the dataset'
    id: str = 'T001'
    name: str = 'DatasetStructureChecker'
    released: str = '23.2.26'
    type: str = 'TranscriptionPlugin'
    version: str = '23.2.26'


class ValidDataSetPlugin:
    info: PluginInfo = PluginInfo()
    errors: List[str] = []
    success_message: str = 'All transcription files and the wavs folder exist'
    error_message: str = 'Detected missing transcription file or "wavs" folder'

    def run(self, path: Path, files: Tuple[Path], dir_name: str) -> None:
        final_messages = []

        for file in (dir_name,) + files:
            if not (Path(path) / str(file)).exists():
                final_messages.append(f'{str(file):>15}')

        if final_messages:
            self.errors = [f'{self.info.id}: {self.error_message}:'] + final_messages
        self.errors = final_messages


def init_plugin() -> ValidDataSetPlugin:
    vds_plugin = ValidDataSetPlugin()
    return vds_plugin



from pathlib import Path
from typing import Dict, List, Tuple


class BaseMeta(type):
    files: Dict[Path, List[Tuple[Path, int, str, str]]] = {}

    def __getitem__(cls, key: str) -> str:
        return getattr(cls, key)

    def __setattr__(cls, name: str, value: str) -> None:
        setattr(cls, name, value)

    def add_file_to_dict(cls, file_path: Path, line_number: int, wav_path: str, transcription: str) -> None:
        file_name = Path(wav_path)
        if file_name not in cls.files:
            cls.files[file_name] = []
        cls.files[file_name].append((file_path, line_number, wav_path, transcription))


class Duplicates(metaclass=BaseMeta):
    files: Dict[str, List[Tuple[Path, int, str, str]]] = {}


class PluginInfo:
    author: str = 'Tadeusz Miszczyk'
    description: str = 'Check if there are any duplicate paths to WAV files in the transcriptions'
    id: str = 'T007'
    name: str = 'DuplicatedTranscriptionChecker'
    released: str = '23.2.26'
    type: str = 'TranscriptionPlugin'
    version: str = '23.2.26'


class ValidDataSetPlugin:
    info: PluginInfo = PluginInfo()
    errors: List[str] = []
    success_message: str = 'No duplicated paths to WAV files found in transcriptions'
    error_message: str = 'Found duplicated path to the WAV file in transcription file'

    def run(  # pylint: disable=too-many-locals
            self, path: Path, files: Tuple[Path], dir_name: str,  # pylint: disable=unused-argument
    ) -> None:
        final_messages = []

        for list_file_name in files:

            if not Path(path / list_file_name).exists():
                final_messages.append(f'ERROR: FILE NOT FOUND IN DATASET: {list_file_name}')
                continue

            file_lines = list(Path(path / list_file_name).read_text(encoding='UTF-8').split('\n'))

            for line_number, line in enumerate(file_lines, start=1):
                if line in ('', '\n\n'):
                    continue
                wav_path, transcription, *_ = line.split('|')
                Duplicates.add_file_to_dict(list_file_name, line_number, wav_path, transcription)

        for _, duplicate in Duplicates.files.items():
            if len(duplicate) > 1:
                full_message = ''
                for entry in duplicate:
                    message = f'{str(entry[0]):>15}: {entry[1]:>6}: {entry[2]}|{entry[3]}'
                    replace_text = (
                        (str(entry[0]), str(entry[0])),
                        (str(entry[1]), str(entry[1])),
                    )
                    for elements in replace_text:
                        message = message.replace(*elements)

                    full_message += f'{message}\n' if full_message == '' else message

                final_messages.append(full_message)

        if final_messages:
            self.errors = [f'{self.info.id}: {self.error_message}:'] + final_messages
        self.errors = final_messages


def init_plugin() -> ValidDataSetPlugin:
    vds_plugin = ValidDataSetPlugin()
    return vds_plugin



from pathlib import Path
from typing import List, Tuple


class PluginInfo:
    author: str = 'Tadeusz Miszczyk'
    description: str = 'Check if all transcriptions end with punctuation marks: ".", "?" or "!"'
    id: str = 'T005'
    name: str = 'PunctuationMarksChecker'
    released: str = '23.2.26'
    type: str = 'TranscriptionPlugin'
    version: str = '23.2.26'


class ValidDataSetPlugin:
    info: PluginInfo = PluginInfo()
    errors: List[str] = []
    success_message: str = 'All transcriptions end with one of the following punctuation marks: ".", "?", or "!"'
    error_message: str = 'Found transcription which does not end with ".", "?" or "!"'

    def run(self, path: Path, files: Tuple[Path], dir_name: str) -> None:  # pylint: disable=unused-argument
        final_messages = []

        for list_file_name in files:

            if not Path(path / list_file_name).exists():
                final_messages.append(f'ERROR: FILE NOT FOUND IN DATASET: {list_file_name}')
                continue

            file_lines = list(Path(path / list_file_name).read_text(encoding='UTF-8').split('\n'))

            for line_number, line in enumerate(file_lines, start=1):
                if line in ('', '\n\n'):
                    continue
                _, transcription, *_ = line.split('|')

                if len(transcription) == 0:
                    final_messages.append(f'{str(list_file_name):>15}: {line_number:>6}: {line}')
                    continue

                if transcription[-1] not in ('.', '?', '!'):
                    final_messages.append(f'{str(list_file_name):>15}: {line_number:>6}: {line}')

        if final_messages:
            self.errors = [f'{self.info.id}: {self.error_message}:'] + final_messages
        self.errors = final_messages


def init_plugin() -> ValidDataSetPlugin:
    vds_plugin = ValidDataSetPlugin()
    return vds_plugin



import argparse
from pathlib import Path
from typing import List


class Config:
    arguments: argparse.Namespace
    disabled_plugins: str = ''
    transcription_plugins_dir_path: Path = Path('transcription')
    files_plugins_dir_path: Path = Path('files')
    activate_plugins: List[str] = []


class ValidDataSetPlugin:
    pass



from pathlib import Path
from typing import List, Tuple

from tqdm import tqdm  # type: ignore


class PluginInfo:
    author: str = 'Tadeusz Miszczyk'
    description: str = 'Check if all files have been added to the transcription files'
    id: str = 'F001'
    name: str = 'WavsTranscriptionChecker'
    released: str = '23.2.26'
    type: str = 'FilePlugin'
    version: str = '23.2.26'


class ValidDataSetPlugin:
    info: PluginInfo = PluginInfo()
    errors: List[str] = []
    success_message: str = 'All WAV files have been added to the transcription files'
    error_message: str = 'Found files that were not added to the transcript files'

    def run(self, path: Path, files: Tuple[Path], dir_name: str) -> None:
        final_messages = []

        list_of_files = list(Path(path / dir_name).glob('*.wav'))
        fixed_list_of_files = []

        for file_path in list_of_files:
            fixed_list_of_files.append(str(file_path).replace(f'{path}/', ''))

        list_of_files_in_transcriptions = []

        for file_list in files:
            if not Path(path / file_list).exists():
                continue

            for line in Path(path / file_list).read_text(encoding='UTF-8').split('\n'):
                wav_path, *_ = line.split('|')
                list_of_files_in_transcriptions.append(wav_path)

        for file in tqdm(fixed_list_of_files):
            if file not in list_of_files_in_transcriptions:
                final_messages.append(f'{file:>44}')

        if final_messages:
            self.errors = [f'{self.info.id}: {self.error_message}:'] + final_messages
        self.errors = final_messages


def init_plugin() -> ValidDataSetPlugin:
    vds_plugin = ValidDataSetPlugin()
    return vds_plugin



import re

from colorama import Fore, Style


class NumbersConfig:
    color: str = Fore.LIGHTGREEN_EX
    style: str = Style.BRIGHT
    regex: str = r'(?:^|\s)(\d+|[0-9]*\.[0-9]+)\D'


class IdConfig:
    color: str = Fore.LIGHTRED_EX
    style: str = Style.BRIGHT
    regex: str = r'([TF][0-9]{3})(?=[\s,: ]|$)'


class PIPEConfig:
    color: str = Fore.YELLOW
    style: str = Style.BRIGHT
    regex: str = r'\|'


class SlashConfig:
    color: str = Fore.RED
    style: str = Style.DIM
    regex: str = r'[\\/]'


class FileConfig:
    color: str = Fore.LIGHTCYAN_EX
    style: str = Style.NORMAL
    regex: str = r'(\S+)\.(\S+)\:'


class ColonConfig:
    color: str = Fore.LIGHTBLUE_EX
    style: str = Style.BRIGHT
    regex: str = r':'


class Colorizer:
    numbers: NumbersConfig = NumbersConfig()
    id: IdConfig = IdConfig()
    pipe: PIPEConfig = PIPEConfig()
    slash: SlashConfig = SlashConfig()
    file: FileConfig = FileConfig()
    colon: ColonConfig = ColonConfig()

    def colorize(self, text: str) -> str:
        colorized_text = ''
        for line in text.split('\n'):
            piped_line = line.split('|')

            new_line = piped_line[0]

            result = re.sub(
                self.numbers.regex, self.numbers.color + self.numbers.style + r'\g<0>' + Style.RESET_ALL, new_line,
            )
            result = re.sub(
                self.id.regex, self.id.color + self.id.style + r'\g<0>' + Style.RESET_ALL, result,
            )
            result = re.sub(
                self.pipe.regex, self.pipe.color + self.pipe.style + r'\g<0>' + Style.RESET_ALL, result,
            )
            result = re.sub(
                self.slash.regex, self.slash.color + self.slash.style + r'\g<0>' + Style.RESET_ALL, result,
            )
            result = re.sub(
                self.file.regex, self.file.color + self.file.style + r'\g<0>' + Style.RESET_ALL, result,
            )
            result = re.sub(
                self.colon.regex, self.colon.color + self.colon.style + r'\g<0>' + Style.RESET_ALL, result,
            )

            colorized_text += f'{Fore.YELLOW}{Style.BRIGHT}|{Style.RESET_ALL}'.join([result] + piped_line[1:]) + '\n'
        return colorized_text



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['vds', 'vds.plugins.files', 'vds.plugins.transcription']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4.6,<0.5.0', 'rich>=13.3.1,<14.0.0', 'tqdm>=4.64.1,<5.0.0']

entry_points = \
{'console_scripts': ['vds = vds.main:main']}

setup_kwargs = {
    'name': 'vds',
    'version': '23.2.27',
    'description': 'ValidDataSet - TTS Lj Speech Dataset Validator',
    'long_description': '# ValidDataSet\n\n<a id="menu"></a>\n\n* [About](#about)\n* [Plugins](#plugins)\n* [Installation](#installation)\n* [Usage](#usage)\n\n## <a id="about"></a>About    <font size="1">[ [Menu](#menu) ]</font>\n\n`ValidDataSet` was created to help validate datasets created based on the Lj Speech Dataset (for Tacotron, Flowtron, Waveglow, or RadTTS).\n\n`VDS` works based on plugins (which can be dynamically added by the user in the future).\n\nDescriptions of current plugins can be found in the [Plugins](#plugins) section.\n\n\n## <a id="plugins"></a>Plugins    <font size="1">[ [Menu](#menu) ]</font>\n\nBelow is a list of currently used plugins (new ones will be added over time).\n\n|  ID  | Name                                | Version | Description                                                               |\n| --- | --- | --- | --- |\n| F001 | WavsTranscriptionChecker            | 23.2.26 | Check if all files have been added to the transcription files             |\n| T001 | DatasetStructureChecker             | 23.2.26 | Check if the "wavs" folder and transcription files exist in the dataset   |\n| T002 | EmptyLineChecker                    | 23.2.26 | Check if there are empty lines in the transcriptions                      |\n| T003 | FilesInTranscriptionChecker         | 23.2.26 | Check if all files added to transcription exist                           |\n| T004 | ExistingWavFileTranscriptionChecker | 23.2.26 | Check if all files added to transcription have a transcription            |\n| T005 | PunctuationMarksChecker             | 23.2.26 | Check if all transcriptions end with punctuation marks: ".", "?" or "!"   |\n| T006 | PunctuationMarksChecker             | 23.2.26 | Check if all lines have the same number of PIPE characters                |\n| T007 | DuplicatedTranscriptionChecker      | 23.2.26 | Check if there are any duplicate paths to WAV files in the transcriptions |\n\n## <a id="installation"></a>Installation    <font size="1">[ [Menu](#menu) ]</font>\n\nTo install ValidDataSet, use the following command:\n\n```shell\npip install vds\n```\n\n## <a id="usage"></a>Usage    <font size="1">[ [Menu](#menu) ]</font>\n\nList of parameters supported by VDS:\n\n```text\n -p, --path            Path to dataset\n -d, --disable         Disable plugins\n -f, --files           Set transcription file names\n     --list-plugins    List plugins\n -v, --verbose         Print additional information\n```\n\nSample commands and their description:\n\nList all plugins:\n```shell\nvds --list-plugins\n```\n\nRun `VDS` with all plugins without additional information:\n```shell\nvds --path /media/username/Disk/Dataset_name/\n```\n\nRun `VDS` with all plugins with additional information:\n```shell\nvds --path /media/username/Disk/Dataset_name/ -v\n```\n\nRun `VDS` without plugins F001,T002,T006 with additional information:\n```shell\nvds --path /media/username/Disk/Dataset_name/ --disable F001,T002,T006 -v\n```\n\nRun `VDS` without plugins F001,T002,T006 with own transcription names and with additional information:\n```shell\nvds --path /media/username/Disk/Dataset_name/ --disable F001,T002,T006 --files train.txt,val.txt -v\n```\n',
    'author': 'Tadeusz Miszczyk',
    'author_email': '42252259+8tm@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'http://github.com/8tm/ValidDataSet',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0',
}


setup(**setup_kwargs)
